#
# Barpanel "Battery Status" plugin
# 
# Copyright (C) 2008 Job and Esther Technologies Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

name = "Battery Status"
description = """Battery status display."""

import barpanel, gtk, gobject, os
import dbus
import dbus.glib
import sys
import gobject

class Battery(gtk.ToggleButton):
    def __init__(self, panel, plugin, bus, udi):
        self.plugin = plugin
        gtk.ToggleButton.__init__(self)
        self.plugin.logINFO(_("Adding battery '%s'" % (udi)))
        self.panel = panel
        self.bus = bus
        self.udi = udi
        self.set_name("EdgeButton")
        self.connect("toggled", self.onToggle)
        self.set_relief(gtk.RELIEF_NONE)
        box = gtk.HBox(False, 2)
        self.add(box)
        self.img = gtk.Image()
        self.previous_icon = ""
        box.add(self.img)
        self.img.show()
        self.bus.add_signal_receiver(self.dbus_handler, 'PropertyModified', 'org.freedesktop.Hal.Device', None, udi)
        self.menuwindow = barpanel.BarpanelMenuWindow("Battery status", "Battery information and charge status", panel.get_image_from_icon_name("battery", 24), panel)
        self.batt_state = gtk.Label()
        self.batt_state.set_padding(8, 8)
        self.batt_state.set_markup("")
        self.batt_state.set_property("xalign", 0)
        self.menuwindow.append(self.batt_state)
        self.batt_charge = gtk.Label()
        self.batt_charge.set_padding(8, 8)
        self.batt_charge.set_markup("")
        self.batt_charge.set_property("xalign", 0)
        self.menuwindow.append(self.batt_charge)

    def update_icon(self, icon_name):
        if self.previous_icon != icon_name:
            self.panel.update_gtk_image(self.img, icon_name)
            self.previous_icon = icon_name

    def update(self):
        dev_obj = self.bus.get_object ('org.freedesktop.Hal', self.udi)
        dev = dbus.Interface (dev_obj, 'org.freedesktop.Hal.Device')
        self.plugin.logDEBUG(_("Percentage: %s" % ( dev.GetProperty('battery.charge_level.percentage'))))
        self.plugin.logDEBUG(_("Discharging: %s" % (dev.GetProperty('battery.rechargeable.is_discharging'))))
        dc = "Running on battery power"
        charge = dev.GetProperty('battery.charge_level.percentage')
        if dev.GetProperty('battery.rechargeable.is_discharging') == 0:
            dc = "Battery charging"
            self.update_icon("battery_charging")
        else:
            if charge <= 25:
                self.update_icon("battery_red")
            elif charge <= 75:
                self.update_icon("battery_yellow")
            else:
                self.update_icon("battery_green")
        self.batt_charge.set_markup("<b>Battery charge:</b>\n%d %%" % (charge))
        self.batt_state.set_markup("<b>Battery status:</b>\n%s" % (dc))
        
    def dbus_handler(self, *args):
        self.plugin.logDEBUG(str(args))
        self.update()

    def onToggle(self, btn):
        self.menuwindow.toggle(btn)

class Plugin(barpanel.Plugin):
    def __init__(self, panel, settings):
        barpanel.Plugin.__init__(self, panel, settings)
        self.bus = dbus.SystemBus ()
        batts = self.getBatteries()
        for b in batts:
            btn = Battery(panel, self, self.bus, b)
            self.add(btn)
            if settings.has_key('desc') and settings['desc'] != None and len(settings['desc'])>0:
                barpanel.tooltips.set_tip(btn, settings['desc'])
            else:
                barpanel.tooltips.set_tip(btn, "Battery status")
            btn.update()
        self.show_all()
    
    def getBatteries(self):
        try:
            hal_obj = self.bus.get_object ('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
            hal = dbus.Interface (hal_obj, 'org.freedesktop.Hal.Manager')
            udis = hal.FindDeviceByCapability ('battery')
            return(udis)
        except Exception, e:
            pass
        return([])
